;************************************************************************
;************************************************************************
;
;	Renesas Electronics     RL78/G10
;
;	CPU	: R5F10Y46
;
;	Clock	: 20MHz
;		Clock Selection: 20MHz
;		Minimum Instruction Execution Time:0.050usec
;
;	Supply Volt. : VDD = 4.5V (@20MHz)
;
;************************************************************************

;************************************************************************
;
; File name : r_init.asm
; Version   : Ver 1.0
; program   : `R
;yz
; Ver 1.0   : 06/05/2015  VK쐬 
; Ver 1.1   : 06/08/2015  WDT @\ǉgp
;
;************************************************************************

;************************************************************************
;
;	for device name and timing definition
;
;************************************************************************


$INCLUDE	(DEV_CH.inc)

;************************************************************************
;
;	End of device name definition
;
;************************************************************************

.EXTERN		main		; main routine
.EXTERN		TDR01DATA	; move timer data

;************************************************************************
;
;
;	interrupt vector table
;
;
;************************************************************************

RESET_START	.VECTOR		0x00000	;(00H) RESET entry

$IFDEF R5F10Y1	
;IRESET		.VECTOR 	0x00004	;(04H) INTWDTI
IRESET		.VECTOR 	0x00006	;(06H) INTP0
IRESET		.VECTOR 	0x00008	;(08H) INTP1
IRESET		.VECTOR 	0x0000A	;(0AH) INTST0/INTCSI00/INTIIC00
IRESET		.VECTOR 	0x0000C	;(0CH) INTSR0
IRESET		.VECTOR 	0x0000E	;(0EH) INTSRE0
IRESET		.VECTOR 	0x00010	;(10H) INTTM01H
IRESET		.VECTOR 	0x00016	;(16H) INTAD
IRESET		.VECTOR 	0x00018	;(18H) INTKR

$ELSEIF R5F10Y4
;IRESET		.VECTOR 	0x00004	;(04H) INTWDTI
IRESET		.VECTOR 	0x00006	;(06H) INTP0
IRESET		.VECTOR 	0x00008	;(08H) INTP1
IRESET		.VECTOR 	0x0000A	;(0AH) INTST0/INTCSI00/INTIIC00
IRESET		.VECTOR 	0x0000C	;(0CH) INTSR0
IRESET		.VECTOR 	0x0000E	;(0EH) INTSRE0
IRESET		.VECTOR 	0x00010	;(10H) INTTM01H
IRESET		.VECTOR 	0x00016	;(16H) INTAD
IRESET		.VECTOR 	0x00018	;(18H) INTKR
IRESET		.VECTOR 	0x0001A	;(1AH) INTP2
IRESET		.VECTOR 	0x0001C	;(1CH) INTP3
IRESET		.VECTOR 	0x0001E	;(1EH) INTTM03H
IRESET		.VECTOR 	0x00020	;(20H) INTIICA0
IRESET		.VECTOR 	0x00022	;(22H) INTTM02
IRESET		.VECTOR 	0x00024	;(24H) INTTM03
IRESET		.VECTOR 	0x00026	;(26H) INTIT
IRESET		.VECTOR 	0x00028	;(28H) INTCMP0

$ENDIF

IRESET		.VECTOR 	0x0007E	;(7EH) BRK
;************************************************************************
;
;	stack area
;
;************************************************************************
DSTK	.DSEG	BASEP
STACKEND:
	.DS	0x20			; Set 32 bytes for stack
STACKTOP:				; Stack top address

;************************************************************************
;
;
;	 declaration directive of constant / variables
;
;
;************************************************************************

XMAIN		.CSEG	UNIT
;************************************************************************
;
;	unexpected interrupt handling
;	(No operation: only exit from interrupt)
;
;************************************************************************
IRESET:
	RETI

XMAIN3		.CSEG
;************************************************************************
;
;	Main routine start
;
;************************************************************************
RESET_START:

	MOV	ES,	#0		; for constant data access

;===============================================================
;	set stack pointer					
;===============================================================
	MOVW	SP,	#LOWW STACKTOP	; set stack pointer to top of RAM

;====================================================================
;	I/O initialize
;====================================================================
	MOV	!PIOR,	#0b00000000	; not use I/O redirection
	CALL	!SINIPORT		; Port initialize
	CALL	!SINICLK		; Clock setting
	CALL    !SINITAU                ; TAU setting
	CLR1	WDTIMK			; enable WDT interval interrupt

	CALL	!main			; buranch to main
MLOOP:
	HALT
	BR	$MLOOP

;====================================================================
SINIPORT:
$IFDEF R5F10Y1
;--------------------------------------------------------------------
;	for 10 pins ( no meaning )
;--------------------------------------------------------------------

	MOV	!PMC0, #0b11100001
			; +++||||+------- must be 1
			;    ||||
			;    ++++--------PMC04-PMC01
			;    		 0 : digital
			;    		 1 : analog

	CLRB    P0                      ; clear port data
	MOV	PM0,	#0b11111111	; set P0 input
	MOV     !PU0,	#0b00000000	; not use pullup
	CLRB	P4			; clear port data
	MOV	PM4,	#0b11111111	; set P4.0 to input

$ELSEIF R5F10Y4
;--------------------------------------------------------------------
;	for 16 pins
;--------------------------------------------------------------------

	MOV	!PMC0, #0b00000001
	                ; |||||||+------- must be 1
			; +++++++--------PMC05-PMC01
			;		 0 : digital
			;		 1 : analog

	CLRB    P0			; set LED off
	MOV	PM0,	#0b11111111	; set LED off
	MOV     !PU0,	#0b00000000 	; not use pullup

	MOV     P4,	#0b00000010	; set LED off
	MOV     PM4,    #0b11111111	; set LED off

$ENDIF

	RET


;====================================================================
;	clock setting for HOCO(20MHz)
;====================================================================
SINICLK:
$IFDEF R5F10Y4
	MOV	CMC,   #0b00000000	; Clock operation mode control register
			; ||+++++|------- must be 0
			; ||     |
			; ||     +------- AMPH
			; ||		[X1 clock oscillation frequency]
			; ||		0 : 2MHzfMX10MHz
			; ||
			; ++------------- EXCLK/OSCSEL
			;		[X1 clock pin operation mode]
			;		00 : Input port mode

	MOV	CSC,   #0b10000000	; Clock operation status control register
			; |++++++|------- must be 0
			; |      |
			; |      +------- HIOSTOP
			; |		 [HOCO clock operation control]
			; |		0 : operating
			; |
			; +-------------- MSTOP
			;		[X1 oscillation mode]
			;		1 : X1 oscillator stopped

	MOV	CKC,   #0b00000000	; System clock control register
			; ++||++++------- must be 0
			;   ||
			;   |+----------- MCM0
			;   |		[Main system clock operation control]
			;   |		0 : select HOCO
			;   |
			;   +------------ MCS <Read Only>
			;		[Status of Main system clock]
			;		0 : HOCO(fIH)
$ENDIF

	MOV	!HOCODIV,#0b00000001	; HOCO frequency select register
			;  +++++|||-------must be 0
			;       |||
			;       +++-------HOCODIV2-0
			;		000 : 20MHz(same as option byte)

	RET

XMAIN4		.CSEG
;====================================================================
;	initialize TAU
;====================================================================
SINITAU:
	SET1	TAU0EN			; enable TAU0
	MOV	!TPS0,	#TPSDATA	; set prescaler
;
;	TM00 initialize
;
	MOV    !TMR00H,	#0b00000000	; timer mode register H
			 ; |++|+|||------- must be 0
			 ; |  | |||
			 ; |  | +++------- STS002-000[trigger control]
			 ; |  |		 000 : soft trigger only
			 ; |  |
			 ; |  +----------- CCS00[count clock select]
			 ; |		  0 : internal clock
			 ; |
			 ; +-------------- CKS001[internal clock select]
			 ;		  0 : select CK00

	MOV    !TMR00L,	#0b00000000	; timer mode register L
                         ; ||++||||------- must be 0
                         ; ||  ||||
                         ; ||  |||+------- MD000[start interrupt]
                         ; ||  |||	  0 : no start interrupt
                         ; ||  |||
                         ; ||  +++-------- MD003-001[operation mode]
                         ; ||		 000 : interval timer mode
                         ; ||
                         ; ++------------- CIS001-000[edge select]
                         ;		00 : falling edge (not used)

	MOV	!TOE0,	#0b00000000	; timer output control
			;  ++++||||------- must be 0
			;      ||||
			;      ++++------- TOE03-00[timer output enable]
			;		0000 : not use timer output
	MOV	!TO0,	#0b00000000	; clear TO

	MOV	TDR00H,	#(COUNT >> 8)	; set high data
	MOV	TDR00L,	#LOW COUNT	; set low data

;
;	TM01 initialize
;
	MOV    !TMR01H,	#0b10000000	; timer mode register H
			 ; |++|||||------- must be 0
			 ; |  |||||
			 ; |  ||+++------- STS012-010[trigger control]
			 ; |  ||	 000 : soft trigger only
			 ; |  ||
			 ; |  |+---------- SPLIT01[8/16bit mode]
			 ; |  |		  0 : 16bit mode
			 ; |  |
			 ; |  +----------- CCS01[count clock select]
			 ; |		  0 : internal clock
			 ; |
			 ; +-------------- CKS011[internal clock select]
			 ;		  1 : select CK01

	MOV    !TMR01L,	#0b00000000	; timer mode register L
                         ; ||++||||------- must be 0
                         ; ||  ||||
                         ; ||  |||+------- MD010[start interrupt]
                         ; ||  |||	  0 : no start interrupt
                         ; ||  |||
                         ; ||  +++-------- MD013-011[operation mode]
                         ; ||		 000 : interval timer mode
                         ; ||
                         ; ++------------- CIS011-010[edge select]
                         ;		00 : falling edge (not used)

	MOVW	AX,	#TDRDT01	; initial data for TM01
	MOVW	TDR01DATA,	AX	; set initial data in buffer

	MOV	TDR01H,	A		; set high data for TM01
	MOV	A,	X
	MOV	TDR01L,	A		; set low data for TM01
;
;	initialize interrupt
;
	CLR1	TMPR100			; set TM00 as level1 interrupt
	CLR1	TMIF00			; clear interrupt request
	CLR1	TMIF01

	RET


	END

